﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.SearchModels;
using ComputersShopContracts.StoragesContracts;
using ComputersShopContracts.ViewModels;
using ComputersShopDataBaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopDataBaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new ComputersShopDataBase();

            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();

                return GetViewModel(element);
            }

            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new ComputersShopDataBase();

            return GetViewModel(context.Orders.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id)));
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }

            using var context = new ComputersShopDataBase();

            return context.Orders.Where(x => x.Id == model.Id).Select(x => GetViewModel(x)).ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new ComputersShopDataBase();

            return context.Orders.Select(x => GetViewModel(x)).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }

            using var context = new ComputersShopDataBase();

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return GetViewModel(newOrder);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new ComputersShopDataBase();

            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();

            return GetViewModel(order);
        }

        private static OrderViewModel GetViewModel(Order order)
        {
            var viewModel = order.GetViewModel;
            using var context = new ComputersShopDataBase();
            var element = context.Computers.FirstOrDefault(x => x.Id == order.ComputerId);
            viewModel.ComputerName = element.ComputerName;
            return viewModel;
        }
    }
}
