﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.BusinessLogicContracts;
using ComputersShopContracts.SearchModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ComputersShopView
{
	public partial class FormCreateOrder : Form
	{
		private readonly ILogger _logger;
		private readonly IComputerLogic _logicC;
		private readonly IOrderLogic _logicO;

		public FormCreateOrder(ILogger<FormCreateOrder> logger, IComputerLogic logicC, IOrderLogic logicO)
		{
			InitializeComponent();
			_logger = logger;
			_logicC = logicC;
			_logicO = logicO;
		}

		private void FormCreateOrder_Load(object sender, EventArgs e)
		{
			_logger.LogInformation("Загрузка компьютеров для заказа");

			try
			{
				var list = _logicC.ReadList(null);
				if (list != null)
				{
					comboBoxComputer.DisplayMember = "ComputerName";
					comboBoxComputer.ValueMember = "Id";
					comboBoxComputer.DataSource = list;
					comboBoxComputer.SelectedItem = null;
				}

			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки списка компьютеров");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void CalcSum()
		{
			if (comboBoxComputer.SelectedValue != null && !string.IsNullOrEmpty(textBoxCount.Text))
			{
				try
				{
					int id = Convert.ToInt32(comboBoxComputer.SelectedValue);
					var Computer = _logicC.ReadElement(new ComputerSearchModel
					{
						Id = id
					});
					int count = Convert.ToInt32(textBoxCount.Text);
					textBoxSum.Text = Math.Round(count * (Computer?.Price ?? 0), 2).ToString();
					_logger.LogInformation("Расчет суммы заказа");
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка расчета суммы заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				}
			}
		}
		private void TextBoxCount_TextChanged(object sender, EventArgs e)
		{
			CalcSum();
		}
		private void comboBoxComputer_SelectedIndexChanged(object sender, EventArgs e)
		{
			CalcSum();
		}
		private void ButtonSave_Click(object sender, EventArgs e)
		{
			if (string.IsNullOrEmpty(textBoxCount.Text))
			{
				MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (comboBoxComputer.SelectedValue == null)
			{
				MessageBox.Show("Выберите компьютер", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			_logger.LogInformation("Создание заказа");
			try
			{
				var operationResult = _logicO.CreateOrder(new OrderBindingModel
				{
					ComputerId = Convert.ToInt32(comboBoxComputer.SelectedValue),
					Count = Convert.ToInt32(textBoxCount.Text),
					Sum = Convert.ToDouble(textBoxSum.Text)
				});
				if (!operationResult)
				{
					throw new Exception("Ошибка при создании заказа. Дополнительная информация в логах.");
			    }
				MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
				DialogResult = DialogResult.OK;
				Close();
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка создания заказа");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ButtonCancel_Click(object sender, EventArgs e)
		{
			DialogResult = DialogResult.Cancel;
			Close();
		}

	}
}
